@@ -20,11 +20,21 @@ my $mt = build_mt("Hello, <?= \$_[0]->{lang} ?> world!\n" x $n);
 
 my $subst_tmpl = qq{Hello, %lang% world!\n} x $n;
 
+my $sprintf_tmpl = qq{Hello, %1\$s world!\n} x $n;
+
 my $vars = {
     lang => 'Template',
 };
 
-$tx->render(undef, $vars) eq $mt->($vars) or die $tx->render($vars);
+{
+    use Test::More;
+    plan tests => 3;
+    my $x = $tx->render(undef, $vars);
+    is $mt->($vars), $x, 'Text::MicroTemplate';
+    (my $o = $subst_tmpl) =~ s/%(\w+)%/$vars->{$1}/g;
+    is $o, $x, 's///g';
+    is sprintf($sprintf_tmpl, $vars->{lang}), $x, 'sprintf';
+}
 
 # suppose PSGI response body
 
@@ -33,7 +43,7 @@ cmpthese -1 => {
         my $body = [$tx->render(undef, $vars)];
         return;
     },
-    mt => sub {
+    TMT => sub {
         my $body = [$mt->($vars)];
         return;
     },
@@ -42,5 +52,9 @@ cmpthese -1 => {
         $body->[0] =~ s/%(\w+)%/$vars->{$1}/g;
         return;
     },
+    'sprintf' => sub {
+        my $body = [ sprintf $sprintf_tmpl, $vars->{lang} ];
+        return;
+    },
 };
 
