@@ -4,7 +4,6 @@ use Any::Moose '::Util::TypeConstraints';
 
 use File::Spec     ();
 use File::Basename ();
-use Encode         ();
 
 with any_moose 'X::Getopt';
 
@@ -29,35 +28,23 @@ has cache => (
 );
 
 has module => (
-    documentation => 'Modules templates will use (e.g. name=sub1,sub2)',
+    documentation => 'Modules templates will use',
     cmd_aliases   => [qw(M)],
     is            => 'ro',
-    isa           => 'HashRef[Str]',
+    isa           => 'ArrayRef[Str]',
     predicate     => 'has_module',
     traits        => $getopt_traits,
 );
 
-has input_encoding => (
-    documentation => 'Input encoding (default: UTF-8)',
-    cmd_aliases   => [qw(ie)],
-    is            => 'rw',
+has input_layer => (
+    documentation => 'Input layer for reading',
+    cmd_aliases   => [qw(l)],
+    is            => 'ro',
     isa           => 'Str',
-    default       => 'UTF-8',
-    predicate     => 'has_input_encoding',
+    predicate     => 'has_input_layer',
     traits        => $getopt_traits,
 );
 
-has output_encoding => (
-    documentation => 'Output encoding (default: UTF-8)',
-    cmd_aliases   => [qw(oe)],
-    is            => 'rw',
-    isa           => 'Str',
-    default       => 'UTF-8',
-    predicate     => 'has_output_encoding',
-    traits        => $getopt_traits,
-);
-
-
 has path => (
     documentation => 'Include paths',
     cmd_aliases   => [qw(I)],
@@ -68,7 +55,7 @@ has path => (
 );
 
 has syntax => (
-    documentation => 'Template syntax (e.g. TTerse)',
+    documentation => 'Template syntax (e.g. "TTerse")',
     cmd_aliases   => [qw(s)],
     is            => 'ro',
     isa           => 'Str',
@@ -76,17 +63,17 @@ has syntax => (
     traits        => $getopt_traits,
 );
 
-has type => (
-    documentation => 'Output content type (html | xml | text)',
-    cmd_aliases   => [qw(t)],
+has escape => (
+    documentation => 'Escaping mode ("html" or "none")',
+    cmd_aliases   => [qw(p)],
     is            => 'ro',
     isa           => 'Str',
-    predicate     => 'has_type',
+    predicate     => 'has_escape',
     traits        => $getopt_traits,
 );
 
 has verbose => (
-    documentation => 'Warning level (default: 2)',
+    documentation => 'Warning level (default to 2)',
     cmd_aliases   => [qw(w)],
     is            => 'ro',
     isa           => 'Str',
@@ -115,7 +102,6 @@ has suffix => (
     cmd_aliases   => [qw(x)],
     is            => 'ro',
     isa           => 'HashRef',
-    default       => sub { +{} },
     traits        => $getopt_traits,
 );
 
@@ -123,8 +109,9 @@ has dest => (
     documentation => 'Destination directry',
     cmd_aliases   => [qw(o)],
     is            => 'ro',
-    isa           => 'Str', # Maybe[Str]
-    required      => 0,
+    isa           => 'Str',
+    lazy          => 1,
+    default       => sub { require Cwd; Cwd::cwd() },
     traits        => $getopt_traits,
 );
 
@@ -138,7 +125,7 @@ has define => (
 );
 
 has eval => (
-    documentation => 'One line of template code',
+    documentation => 'One line of templates',
     cmd_aliases   => [qw(e)],
     is            => 'ro',
     isa           => 'Str',
@@ -171,40 +158,30 @@ has version => (
 );
 
 sub run {
-    my($self, @targets) = @_;
+    my $self = shift;
+
+    my $targets = $self->extra_argv;
 
     my %args;
     foreach my $field qw(
-        cache_dir cache path syntax
-        type verbose
+        cache_dir cache module input_layer path syntax
+        escape verbose
             ) {
         my $method = "has_$field";
         $args{ $field } = $self->$field if $self->$method;
     }
-    if($self->has_module) { # re-mapping
-        my $mod = $self->module;
-        my @mods;
-        while(my $name = each %{$mod}) {
-            push @mods, $name, [ split /,/, $mod->{$name} ];
-        }
-        $args{module} = \@mods;
-    }
-
-    if(my $ie = $self->input_encoding) {
-        $args{input_layer} = ":encoding($ie)";
-    }
 
     local $ENV{XSLATE} = $self->debug
         if $self->has_debug;
 
     require Text::Xslate;
+    Any::Moose::load_class($self->engine);
 
     if($self->version) {
         print $self->version_info(), "\n";
         return;
     }
 
-    Any::Moose::load_class($self->engine);
     my $xslate = $self->engine->new(%args);
 
     if($self->has_eval) {
@@ -212,13 +189,12 @@ sub run {
         if($self->has_define){
             %vars = %{$self->define};
         }
-        $vars{ARGV} = \@targets;
-        $vars{ENV}  = \%ENV;
+        $vars{ARGV} = $targets;
         print $xslate->render_string($self->eval, \%vars), "\n";
         return;
     }
 
-    foreach my $target (@targets) {
+    foreach my $target (@$targets) {
         # XXX if you have a directory, just pushed that into the list of
         # path in the xslate object
         if ( -d $target ) {
@@ -270,39 +246,29 @@ sub process_file {
         splice @comps, 0, $base;
         $filearg = File::Spec->catfile( @comps, File::Basename::basename($file) );
     }
+    my $outfile = File::Spec->catfile( $dest, $filearg );
+    if ($suffix_map && (my $replace = $suffix_map->{ $suffix })) {
+        $outfile =~ s/$suffix$/$replace/;
+    }
 
-    my $outfile;
-
-    if(defined $dest or exists $suffix_map->{$suffix}) {
-        $outfile= File::Spec->catfile( $dest, $filearg );
-        if (my $replace = $suffix_map->{ $suffix }) {
-            $outfile =~ s/$suffix$/$replace/;
-        }
-
-        my $dir = File::Basename::dirname( $outfile );
-        if (! -d $dir) {
-            require File::Path;
-            if (! File::Path::mkpath( $dir )) {
-                die "Could not create directory $dir: $!";
-            }
+    my $dir = File::Basename::dirname( $outfile );
+    if (! -d $dir) {
+        require File::Path;
+        if (! File::Path::mkpath( $dir )) {
+            die "Could not create directory $dir: $!";
         }
     }
 
+
     my $rendered = $xslate->render( $filearg, $self->define );
-    $rendered = Encode::encode($self->output_encoding, $rendered);
 
-    if(defined $outfile) {
-        my $fh;
-        open( $fh, '>', $outfile )
-            or die "Could not open file $outfile for writing: $!";
+    my $fh;
+    open( $fh, '>', $outfile )
+        or die "Could not open file $outfile for writing: $!";
 
-        print $fh $rendered;
+    print $fh $rendered;
 
-        close $fh or warn "Could not close file $outfile: $!";
-    }
-    else {
-        print $rendered;
-    }
+    close $fh or warn "Could not close file $outfile: $!";
 }
 
 sub version_info {
@@ -330,8 +296,9 @@ This is the guts of C<xslate(1)>.
 
 =head1 AUTHOR
 
-This is originally written by Maki, Daisuke (lestrrat),
-and also maintained by Fuji, Goro (gfx),
+Maki, Daisuke (lestrrat)
+
+Fuji, Goro (gfx)
 
 =head1 SEE ALSO
 
